#!/usr/bin/perl -w
use strict;

###############################################################
#        Users edit this section only                         #
#
#Wrfout file path and name
my $wrfout='./wrfout_d01_2000-01-24_12:00:00_PLEV';
#Dir to dump images into
my $graphics_dir="./";
#Date output frequency in hrs
my $freq=3;
#Duration of run in hrs
my $duration=12;
#Initialization year
my $begyyyy=2000;
#Initialization month
my $begmon="jan";
#Initialization day
my $begdd=24;
#Initialization hour
my $beghh=12;

#WRF domain info from namelist
my $isize=73;
my $jsize=60;
my $reflat=34.83001;
my $reflon=-81.03;
my $truelat1=30.0;
my $truelat2=60.0;
my $standlon=-98.0;
my $dx=30000;
my $dy=30000;

##############################################################

$ENV{GASCRP} = "/usr/local/grads/lib ./";

#figure out ctl file stuff
my $newdset="dset $wrfout";
my $numt=($duration/$freq)+1;
my $newtdef="tdef $numt linear ${beghh}Z${begdd}${begmon}${begyyyy} ${freq}hr";
my $iref=$isize/2;
my $jref=$jsize/2;
my $newpdef= "pdef $isize $jsize lccr $reflat $reflon $iref $jref $truelat1 $truelat2 $standlon $dx $dy";
#approximate grads display res based on wrf res
my $xres=$dx/100000;
my $yres=$dy/100000;
my $numlondeg=((($isize-1)*$dx)/(3.14159*2*6378100))*360;
#When displaying in LatLon proj, fills display are mostly
my $xadj=$numlondeg*0.1;
my $numx=($numlondeg/$xres)-(2*$xadj/$xres);
$numx=int($numx);
#my $latdegsize=$yres*($jsize-1);
my $numlatdeg=(($jsize-1)*$dy)/111000;
my $yadj=$numlatdeg*0.1;
my $numy=($numlatdeg/$yres)-(2*$yadj/$yres);
$numy=int($numy);
my $startlon=$reflon-($numlondeg/2)+$xadj;
#my $startlon=$reflon-($londegsize/2)+$xres*9;
#my $startlat=$reflat-($latdegsize/2)+$yres*9;
my $startlat=$reflat-($numlatdeg/2)+$yadj;
my $newxdef = "xdef $numx linear $startlon $xres";
my $newydef = "ydef $numy linear $startlat $yres";

#Prompt user for type of plot
print "Type of plot? (options are 70pcp, Thickness, Theta, or 30kPa)\n";
my $ptype=<>;
&trim($ptype);
if (($ptype ne "Thickness")&&($ptype ne "70pcp")&&($ptype ne "30kPa")&&($ptype ne "Theta")){
    die "Invalid. Please carefully specify plot type\n";
}

#Edit ctl file
open(CTL, "<\./template.ctl") or die ("Can't find template.ctl, make sure its in the same dir as graphics.pl\n");
my @ctl=<CTL>;
close(CTL);
my $ctlline;
foreach $ctlline (@ctl){
    if ($ctlline =~ /^dset.+$/){
        $ctlline =~ s/^dset.+$/$newdset/g;
    }
    if ($ctlline =~ /^tdef.+$/){
        $ctlline =~ s/^tdef.+$/$newtdef/g;
    }
    if ($ctlline =~ /^pdef.+$/){
        $ctlline =~ s/^pdef.+$/$newpdef/g;
    }
    if ($ctlline =~ /^xdef.+$/){
        $ctlline =~ s/^xdef.+$/$newxdef/g;
    }
    if ($ctlline =~ /^ydef.+$/){
        $ctlline =~ s/^ydef.+$/$newydef/g;
    }
}

open(NEWCTL, ">\./wrfout.ctl") or die ("Can't write new ctl file\n");
print NEWCTL (@ctl);
close(NEWCTL);

#Make some graphics!
my $lasttstep=($duration/$freq)+1;
my $tstep=1;
my @output;
while ($tstep <= $lasttstep){
    open(WRAP, ">plotter.gs") or die ("Can't open plotter.gs for writing\n");
    print "tstep $tstep\n";
    #print WRAP "rc = gsfallow(\"on\")\n_tstep=$tstep\n\'plot$ptype.gsf\'\n";
	print WRAP "rc = gsfallow(\"on\")\n_tstep=$tstep\n_ptype=\"$ptype\"\nplot$ptype()\n\'close 1\'\n\'quit\'\n";
    #print WRAP "_tstep=$tstep+1\n";
	close(WRAP);

    @output=`grads -blc ./plotter.gs 1>&1 2>&1`;
    print "@output\n";

    $tstep++;
}

#Trim
sub trim{
    $_[0]=~s/^\s+//;
    $_[0]=~s/\s+$//;
    return;
}
